@@ -259,7 +259,9 @@ def do_add(
     deps_to_update = group_deps if unconstrained else requirements
     save_version_specifiers({group: deps_to_update}, resolved, save)
     if not dry_run:
-        project.add_dependencies(deps_to_update, group, dev)
+        project.add_dependencies(
+            deps_to_update, group, dev, replace_editable=no_editable
+        )
         project.write_lockfile(project.lockfile, False)
 
     if sync:
@@ -342,7 +344,7 @@ def do_update(
     reqs = [r for deps in all_dependencies.values() for r in deps.values()]
     resolved = do_lock(
         project,
-        strategy if top or packages else "all",
+        strategy,
         chain.from_iterable(updated_deps.values()),
         reqs,
         dry_run=dry_run,
