@@ -80,9 +80,7 @@ class BaseRepository:
             # XXX: If the requirement has extras, add the original candidate
             # (without extras) as its dependency. This ensures the same package with
             # different extras resolve to the same version.
-            self_req = dataclasses.replace(
-                candidate.req, extras=None, marker=None, specifier=get_specifier("")
-            )
+            self_req = dataclasses.replace(candidate.req, extras=None, marker=None)
             reqs.append(self_req)
         # Store the metadata on the candidate for caching
         candidate.requires_python = requires_python
@@ -213,13 +211,16 @@ class BaseRepository:
             matching_candidates: Iterable[Candidate] = [candidate]
         else:
             matching_candidates = self.find_candidates(req, True)
+        result: dict[str, str] = {}
         with self.environment.get_finder(self.sources) as finder:
             self._hash_cache.session = finder.session  # type: ignore
-            return {
-                c.link.filename: self._hash_cache.get_hash(c.link)  # type: ignore
-                for c in matching_candidates
-                if c.link and not c.link.is_vcs
-            } or None
+            for c in matching_candidates:
+                link = c.prepare(self.environment).ireq.link
+                if not link or link.is_vcs:
+                    continue
+                result[link.filename] = self._hash_cache.get_hash(link)
+
+        return result or None
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
         """Return an iterable of getter functions to get dependencies, which will be
@@ -392,7 +393,7 @@ class LockedRepository(BaseRepository):
         url = getattr(candidate.req, "url", None)
         return (
             candidate.identify(),
-            candidate.version,
+            candidate.version if not url else None,
             url_without_fragments(url) if url else None,
             candidate.req.editable,
         )
@@ -401,7 +402,7 @@ class LockedRepository(BaseRepository):
         return self.candidate_info[self._identify_candidate(candidate)]
 
     def dependency_generators(self) -> Iterable[Callable[[Candidate], CandidateInfo]]:
-        return (self._get_dependencies_from_cache, self._get_dependencies_from_lockfile)
+        return (self._get_dependencies_from_lockfile,)
 
     def get_dependencies(
         self, candidate: Candidate
